<?php

/**
 * @file
 * Contains \Drupal\forward\Form\ForwardForm.
 */

namespace Drupal\forward\Form;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Flood\FloodInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AnonymousUserSession;
use Drupal\Core\Url;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Utility\Unicode;

/**
 * Forward a page to a friend
 */
class ForwardForm extends FormBase {
  protected $entity;
  protected $settings;

  /**
   * Constructs a Forward Form.
   */
  public function __construct($entity) {
    $this->entity = $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'forward_form_' . $this->entity->getEntityTypeId() . '_' . $this->entity->id();
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['forward.form'];
  }

  /**
   * Get a token
   */
  private function getToken(FormStateInterface $form_state = NULL) {
    $token = array();
    if ($form_state && $form_state->getValue('name')) {
      $token = ['forward' => ['sender_name' => $form_state->getValue('name')]];
    } elseif ($this->currentUser()->isAuthenticated()) {
      $token = ['forward' => ['sender_name' => $this->currentUser()->getDisplayName()]];
    }
    if ($form_state && $form_state->getValue('email')) {
      $token['forward']['sender_email'] = $form_state->getValue('email');
    } elseif ($this->currentUser()->isAuthenticated()) {
      $token['forward']['sender_email'] = $this->currentUser()->getEmail();
    }
    if ($form_state) {
      $token['forward']['entity'] = $form_state->get('#entity');
    }
    // Allow other modules to add more tokens
    if ($extra_tokens = \Drupal::moduleHandler()->invokeAll('forward_token', array($form_state))) {
      $token += $extra_tokens;
    }
    return $token;
  }

  /**
   * Get the event name used for Flood control
   */
  private function getFloodControlEventName() {
    return 'forward.send';
  }

  /**
   * Determine if a given display is valid for an entity
   */
  private function isValidDisplay(EntityInterface $entity, $view_mode) {
    // Assume the display is valid
    $valid = TRUE;

    // Build display name
    if ($entity->getEntityType()->hasKey('bundle')) {
      // Bundled entity types, e.g. node
      $display_name = $entity->getEntityTypeId() . '.' . $entity->bundle() . '.' . $view_mode;
    } else {
      // Entity types without bundles, e.g. user
      $display_name = $entity->getEntityTypeId() . '.' . $view_mode;
    }

    // Attempt load
    $display = \Drupal::entityTypeManager()->getStorage('entity_view_display')->load($display_name);
    if ($display) {
      // If the display loads, it exists in configuration, and status can be checked
      $valid = FALSE;
      if ($display->status()) {
        $valid = TRUE;
      }
    }

    return $valid;
  }

  /**
   * Logging
   */
  private function logEvent(EntityInterface $entity) {
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $entity_id = $entity->id();

    $uid = $this->currentUser()->id();
    $path = substr($entity->toUrl()->toString(), 1);
    $ip_address = \Drupal::request()->getClientIp();
    $timestamp = REQUEST_TIME;

    // Insert into log
    \Drupal::database()->insert('forward_log')
      ->fields(array(
        'type' => $entity_type,
        'id' => $entity_id,
        'path' => $path,
        'action' => 'SENT',
        'timestamp' => $timestamp,
        'uid' => $uid,
        'hostname' => $ip_address,
      ))
      ->execute();

   // Update statistics
   \Drupal::database()->merge('forward_statistics')
      ->key(array(
        'type' => $entity_type,
        'bundle' =>  $bundle,
        'id' => $entity_id,
      ))
      ->fields(array(
        'forward_count' => 1,
        'last_forward_timestamp' => $timestamp,
      ))
      ->expression('forward_count', 'forward_count + 1')
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, array $settings = NULL) {
    if (!$settings) {
      $settings = $this->config('forward.settings')->get();
    }
    $this->settings = $settings;
    $form_state->set('#entity', $this->entity);
    $token_service = \Drupal::token();
    $token = $this->getToken($form_state);
    $langcode = $this->entity->language()->getId();

    // Build the form
    if ($settings['forward_interface_type'] == 'link') {
      // Set the page title dynamically
      $form['#title'] = $token_service->replace($settings['forward_link_title'], $token, array('langcode' => $langcode));
    } else {
      // Inline form
      $form['message'] = array(
        '#type' => 'details',
        '#title' => $token_service->replace($settings['forward_link_title'], $token, array('langcode' => $langcode)),
        '#description' => '',
        '#open' => FALSE,
        '#weight' => $settings['forward_interface_weight'],
      );
    }
    $form['message']['instructions'] = array(
      '#markup' => $token_service->replace($settings['forward_form_instructions'], $token, array('langcode' => $langcode)),
    );
    $form['message']['email'] = array(
      '#type' => 'email',
      '#title' => t('Your email address'),
      '#maxlength' => 254,
      '#required' => TRUE,
    );
    $form['message']['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Your name'),
      '#maxlength' => 128,
      '#required' => TRUE,
    );
    $form['message']['recipient'] = array(
      '#type' => 'email',
      '#title' => t('Send to'),
      '#maxlength' => 254,
      '#description' => t('Enter the email address of the recipient.'),
      '#required' => TRUE,
    );
    if ($settings['forward_form_display_page']) {
      $form['message']['page'] = array(
        '#type' => 'item',
        '#title' => t('You are going to email the following:'),
        '#markup' => \Drupal::l($this->entity->label(), $this->entity->toUrl()),
      );
    }
    if ($settings['forward_form_display_subject']) {
      $form['message']['subject'] = array(
        '#type' => 'item',
        '#title' => t('The message subject will be:'),
        '#markup' => $token_service->replace($settings['forward_email_subject'], $token, array('langcode' => $langcode)),
      );
    }
    if ($settings['forward_form_display_body']) {
      $form['message']['body'] = array(
        '#type' => 'item',
        '#title' => t('The introductory message body will be:'),
        '#markup' => $token_service->replace($settings['forward_email_message'], $token, array('langcode' => $langcode)),
      );
    }
    if ($settings['forward_personal_message']) {
      $form['message']['message'] = array(
        '#type' => 'textarea',
        '#title' => t('Your personal message'),
        '#default_value' => '',
        '#cols' => 50,
        '#rows' => 5,
        '#description' => $settings['forward_personal_message_filter'] ? t('These HTML tags are allowed in this field: @tags.', array('@tags' => $settings['forward_personal_message_tags'])) : t('HTML is not allowed in this field.'),
        '#required' => ($settings['forward_personal_message'] == 2),
      );
    }

    // Submit button
    if ($settings['forward_interface_type'] == 'form') {
      // When using a collapsible form, move submit button into fieldset
      $form['message']['actions'] = array('#type' => 'actions');
      $form['message']['actions']['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Send Message'),
      );
    } else {
      // When using a separate form page, use actions directly
      $form['actions'] = array('#type' => 'actions');
      $form['actions']['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Send Message'),
      );
    }

    // Default name and email address to logged in user
    if ($this->currentUser()->isAuthenticated()) {
      if ($this->currentUser()->hasPermission('override email address')) {
        $form['message']['email']['#default_value'] = $this->currentUser()->getEmail();
      } else {
        // User not allowed to change sender email address
        $form['message']['email']['#type'] = 'hidden';
        $form['message']['email']['#value'] = $this->currentUser()->getEmail();
      }
      $form['message']['name']['#default_value'] = $this->currentUser()->getDisplayName();
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$this->currentUser()->hasPermission('override flood control')) {
      $event = $this->getFloodControlEventName();
      $flood = \Drupal::flood();
      if (!$flood->isAllowed($event, $this->settings['forward_flood_control_limit'])) {
        $message = new FormattableMarkup($this->settings['forward_flood_control_error'], ['@number' => $this->settings['forward_flood_control_limit']]);
        $form_state->setErrorByName('', $message);
      }
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get form values
    $entity = $form_state->get('#entity');
    $recipient = $form_state->getValue('recipient');

    // Use the entity language to drive translation
    $langcode = $entity->language()->getId();

    // Switch to anonymous user session if logged in, unless bypassing access control
    $switched = FALSE;
    if ($this->currentUser()->isAuthenticated() && empty($this->settings['forward_bypass_access_control'])) {
      $accountSwitcher = \Drupal::service('account_switcher');
      $accountSwitcher->switchTo(new AnonymousUserSession());
      $switched = TRUE;
    }

    try {
      // Build the message subject line
      $token_service = \Drupal::token();
      $token = $this->getToken($form_state);
      $params['subject'] = $token_service->replace($this->settings['forward_email_subject'], $token, array('langcode' => $langcode));

      // Build the entity content
      $view_mode = '';
      $elements = array();
      if ($entity->access('view')) {
        $view_builder = \Drupal::entityTypeManager()->getViewBuilder($entity->getEntityTypeId());
        $view_mode = 'forward';
        if ($this->isValidDisplay($entity, $view_mode)) {
          $elements = $view_builder->view($entity, $view_mode, $langcode);
        }
        if (empty($elements)) {
          $view_mode = 'teaser';
          if ($this->isValidDisplay($entity, $view_mode)) {
            $elements = $view_builder->view($entity, $view_mode, $langcode);
          }
        }
        if (empty($elements)) {
          $view_mode = 'full';
          $elements = $view_builder->view($entity, $view_mode, $langcode);
        }
      }
      $elements['#forward_build'] = TRUE; // prevent recursion
      $content = \Drupal::service('renderer')->render($elements);

      // Build the header line
      $header = ['#markup' => $token_service->replace($this->settings['forward_email_message'], $token, array('langcode' => $langcode))];

      // Build the personal message if present
      $message = '';
      if ($this->settings['forward_personal_message']) {
        if ($this->settings['forward_personal_message_filter']) {
          // HTML allowed in personal message, so filter out anything but the allowed tags
          $raw_values = $form_state->getUserInput();
          $allowed_tags = explode(',', $this->settings['forward_personal_message_tags']);
          $message = !empty($raw_values['message']) ? Xss::filter($raw_values['message'], $allowed_tags) : '';
          $message = ['#markup' => nl2br($message)];
        } else {
          // HTML not allowed in personal message, so use the sanitized version converted to plain text
          $message = ['#plain_text' => nl2br($form_state->getValue('message'))];
        }
      }

      // Build the email body
      $render_array = array(
        '#theme' => 'forward',
        '#email' => $form_state->getValue('email'),
        '#header' => $header,
        '#message' => $message,
        '#settings' => $this->settings,
        '#entity' => $entity,
        '#content' => $content,
        '#view_mode' => $view_mode,
      );

      // Allow modules to alter the render array for the message
      \Drupal::moduleHandler()->alter('forward_mail_pre_render', $render_array, $form_state);

      // Render the message
      $params['body'] =  \Drupal::service('renderer')->render($render_array);

      // Apply filters such as Pathologic for link correction
      if ($this->settings['forward_filter_format']) {
        // This filter was setup by the Forward administrator for this purpose only, whose permission to run the filter was checked at that time
        // Therefore, no need to check filter access again here
        $params['body'] = check_markup($params['body'], $this->settings['forward_filter_format'], $langcode);
      }

      // Allow modules to alter the final message body
      \Drupal::moduleHandler()->alter('forward_mail_post_render', $params['body'], $form_state);
    } catch (Exception $e) {
      if ($switched) {
        $accountSwitcher->switchBack();
        $switched = FALSE;
      }
      \Drupal::logger('forward')->error($e->getMessage());
    }

    // Switch back to logged in user if necessary
    if ($switched) {
      $accountSwitcher->switchBack();
    }

    // Build the from email address and Reply-To
    $from = $this->settings['forward_email_from_address'];
    if (empty($from)) {
      $from = \Drupal::config('system.site')->get('mail');
    }
    if (empty($from)) {
      $site_mail = ini_get('sendmail_from');
    }
    $params['headers']['Reply-To'] = trim(Unicode::mimeHeaderEncode($form_state->getValue('name')) . ' <' . $form_state->getValue('email') . '>');

    // Send the email to the recipient
    $key = 'send_entity';
    $mail_service = \Drupal::service('plugin.manager.mail');
    $mail_service->mail('forward', $key, $recipient, $langcode, $params, $from);

    // Log this for tracking purposes
    $this->logEvent($entity);

    // Register event for flood control
    $event = $this->getFloodControlEventName();
    $flood = \Drupal::flood();
    $flood->register($event);

    // Display a confirmation message
    $message = $token_service->replace($this->settings['forward_form_confirmation'], $token, array('langcode' => $langcode));
    if ($message) {
      drupal_set_message($message);
    }

    // Redirect back to entity page
    if ($this->settings['forward_interface_type'] == 'link') {
      $form_state->setRedirectUrl($entity->toUrl());
    }
  }
}
