<?php

/**
 * @file
 * Contains \Drupal\forward\Form\SettingsForm.
 */

namespace Drupal\forward\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure settings for this module.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'forward_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['forward.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $forward_config = $this->config('forward.settings');
    $settings = $forward_config->get();

    // Entity Types
    $form['forward_entities'] = array(
      '#type' => 'fieldset',
      '#title' => t('Entity Types'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );
    $entity_types = \Drupal::entityTypeManager()->getDefinitions();
    $options = array();
    $defaults = array();
    foreach ($entity_types as $type => $info) {
      if (is_a($info, 'Drupal\Core\Entity\ContentEntityType')) {
        // Filter some entity types out
        if (!in_array($type, array('block_content', 'contact_message', 'file', 'menu_link_content', 'node', 'shortcut'))) {
          $options[$type] = $info->getLabel();
          if ($settings['forward_entity_' . $type]) {
            $defaults[] = $type;
          }
        }
      }
    }
    $form['forward_entities']['forward_entity_types'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Entity types'),
      '#options' => $options,
      '#default_value' => $defaults,
      '#description' => t('Choose entity types to show Forward on.'),
    );
    // Bundles
    foreach ($entity_types as $type => $info) {
      if (is_a($info, 'Drupal\Core\Entity\ContentEntityType')) {
        if ($settings['forward_entity_' . $type] || ($type == 'node')) {
          $bundles = \Drupal::entityManager()->getBundleInfo($type);
          if (count($bundles) == 1) {
            foreach ($bundles as $bundle => $bundle_info) {
              $form['forward_entities']['forward_' . $type . '_types'] = array(
                '#type' => 'hidden',
                '#value' => $bundle,
              );
            }
          } else {
            $options = array();
            $defaults = array();
            foreach ($bundles as $bundle => $bundle_info) {
              $options[$bundle] = $bundle_info['label'];
              if ($settings['forward_' . $type . '_' . $bundle]) {
                $defaults[] = $bundle;
              }
            }
            $form['forward_entities']['forward_' . $type . '_types'] = array(
              '#type' => 'checkboxes',
              '#title' => t($info->getLabel() . ' bundles'),
              '#options' => $options,
              '#default_value' => $defaults,
              '#description' => t('Choose @type bundles to show Forward on.', array('@type' => $info->getLowercaseLabel())),
            );
          }
        }
      }
    }
    // View Modes
    $modes = array('full' => t('Full entity'), 'teaser' => t('Teaser'));
    $options = array();
    $defaults = array();
    foreach ($modes as $mode => $info) {
      $options[$mode] = $info;
      if ($settings['forward_view_' . $mode]) {
        $defaults[] = $mode;
      }
    }
    $form['forward_entities']['forward_view_modes'] = array(
      '#type' => 'checkboxes',
      '#title' => t('View modes'),
      '#options' => $options,
      '#default_value' => $defaults,
      '#description' => t('Choose view modes to show Forward on.'),
    );
    // Interface
    $form['forward_interface'] = array(
      '#type' => 'details',
      '#title' => t('Interface'),
      '#open' => false,
    );
    $form['forward_interface']['forward_interface_type'] = array(
      '#type' => 'select',
      '#title' => t('Interface type'),
      '#default_value' => $settings['forward_interface_type'],
      '#options' => array('link' => t('Link to separate page'), 'form' => t('Display inline form')),
      '#description' => t('Choose how the Forward form is reached from the displayed entity. Inline forms are displayed within a collapsible fieldset on the entity being forwarded.'),
    );
    $form['forward_interface']['forward_interface_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title of interface element'),
      '#default_value' => $settings['forward_interface_title'],
      '#description' => t('Set the text of the link or fieldset title. Replacement tokens may be used.'),
      '#required' => true,
    );
    $form['forward_interface']['forward_interface_weight'] = array(
      '#type' => 'number',
      '#title' => t('Weight'),
      '#default_value' => $settings['forward_interface_weight'],
      '#description' => t('Set the weight of the link or inline form for positioning.'),
      '#required' => true,
    );
    $form['forward_interface']['forward_link_options'] = array(
      '#type' => 'fieldset',
      '#title' => t('Link Options'),
      '#states' => array(
        'visible' => array(
          ':input[name=forward_interface_type]' => array('value' => 'link'),
        ),
      ),
    );
    $form['forward_interface']['forward_link_options']['forward_link_inline'] = array(
      '#type' => 'checkbox',
      '#title' => t('Place the link inline with other node links'),
      '#default_value' => $settings['forward_link_inline'],
      '#description' => t('Include with inline links like "Read more" and "Add comment". If not placed inline, or the entity is not a node, the link is added to the content area.'),
    );
    $form['forward_interface']['forward_link_options']['forward_link_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Page title'),
      '#default_value' => $settings['forward_link_title'],
      '#maxlength' => 255,
      '#description' => t('Page title for the Forward form page.'),
      '#required' => true,
    );
    $form['forward_interface']['forward_link_options']['forward_link_style'] = array(
      '#type' => 'radios',
      '#title' => t('Style'),
      '#default_value' => $settings['forward_link_style'],
      '#options' => array(0 => t('Text only'), 1 => t('Icon only'), 2 => t('Icon and text')),
      '#description' => t('Select the visual style of the link.'),
    );
    $form['forward_interface']['forward_link_options']['forward_link_icon'] = array(
      '#type' => 'textfield',
      '#title' => t('Path to custom icon'),
      '#default_value' => $settings['forward_link_icon'],
      '#description' => t('The path to your custom link icon instead of the default icon. Example: sites/default/files/icon.png'),
    );
    $form['forward_interface']['forward_link_options']['forward_link_noindex'] = array(
      '#type' => 'checkbox',
      '#title' => t('Generate a noindex meta tag on the forward page') . ' (name="robots", content="noindex, nofollow")',
      '#default_value' => $settings['forward_link_noindex'],
    );
    $form['forward_interface']['forward_link_options']['forward_link_nofollow'] = array(
      '#type' => 'checkbox',
      '#title' => t('Generate a nofollow tag on the forward link (rel="nofollow")'),
      '#default_value' => $settings['forward_link_nofollow'],
    );
    // Forward Form
    $form['forward_form'] = array(
      '#type' => 'details',
      '#title' => t('Forward Form'),
      '#open' => false,
    );
    $form['forward_form']['forward_form_instructions'] = array(
      '#type' => 'textarea',
      '#title' => t('Instructions'),
      '#default_value' => $settings['forward_form_instructions'],
      '#rows' => 5,
      '#description' => t('The instructions to display above the form.  Replacement tokens may be used.'),
    );
    $form['forward_form']['form_display_options'] = array(
      '#type' => 'fieldset',
      '#title' => t('Form Fields'),
    );
    $form['forward_form']['form_display_options']['forward_form_display_page'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display a link to the page being forwarded'),
      '#default_value' => $settings['forward_form_display_page'],
    );
    $form['forward_form']['form_display_options']['forward_form_display_subject'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display the email message subject'),
      '#default_value' => $settings['forward_form_display_subject'],
    );
    $form['forward_form']['form_display_options']['forward_form_display_body'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display the email message body'),
      '#default_value' => $settings['forward_form_display_body'],
    );
    $form['forward_form']['personal_messages'] = array(
      '#type' => 'fieldset',
      '#title' => t('Personal Message Field'),
    );
    $form['forward_form']['personal_messages']['forward_personal_message'] = array(
      '#type' => 'select',
      '#title' => t('Personal message'),
      '#options' => array(0 => 'Hidden', 1 => 'Optional', 2 => 'Required'),
      '#default_value' => $settings['forward_personal_message'],
      '#description' => t('Choose whether the personal message field on the form will be hidden, optional or required.'),
    );
    $form['forward_form']['personal_messages']['forward_personal_message_filter'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow HTML in personal messages'),
      '#return_value' => 1,
      '#default_value' => $settings['forward_personal_message_filter'],
      '#description' => t('Filter XSS and all tags not allowed below from the personal message.  Otherwise any HTML in the message will be converted to plain text.'),
      '#states' => array(
        'invisible' => array(
          ':input[name=forward_personal_message]' => array('value' => 0),
        ),
      ),
    );
    $form['forward_form']['personal_messages']['forward_personal_message_tags'] = array(
      '#type' => 'textfield',
      '#title' => t('Allowed HTML tags'),
      '#default_value' => $settings['forward_personal_message_tags'],
      '#description' => t('List of allowed tags (separated by commas) that will be allowed if HTML is enabled above.  The default tags allowed are: p,br,em,strong,cite,code,ul,ol,li,dl,dt,dd'),
      '#states' => array(
        'invisible' => array(
          ':input[name=forward_personal_message]' => array('value' => 0),
        ),
      ),
    );
    $form['forward_form']['forward_form_confirmation'] = array(
      '#type' => 'textarea',
      '#title' => t('Confirmation message'),
      '#default_value' => $settings['forward_form_confirmation'],
      '#rows' => 5,
      '#description' => t('The thank you message displayed after the user successfully submits the form.  Replacement tokens may be used.'),
    );
    // Defaults for Message to Recipient
    $form['forward_email_defaults'] = array(
      '#type' => 'details',
      '#title' => t('Forward Email'),
      '#open' => false,
    );
    $form['forward_email_defaults']['forward_email_logo'] = array(
      '#type' => 'textfield',
      '#title' => t('Path to custom logo'),
      '#default_value' => $settings['forward_email_logo'],
      '#maxlength' => 256,
      '#description' => t('The path to the logo you would like to use instead of the site default logo. Example: sites/default/files/logo.png'),
      '#required' => false,
    );
    $form['forward_email_defaults']['forward_email_from_address'] = array(
      '#type' => 'email',
      '#title' => t('From address'),
      '#default_value' => $settings['forward_email_from_address'],
      '#maxlength' => 254,
      '#required' => true,
    );
    $form['forward_email_defaults']['forward_email_subject'] = array(
      '#type' => 'textfield',
      '#title' => t('Subject line'),
      '#default_value' => $settings['forward_email_subject'],
      '#maxlength' => 256,
      '#description' => t('Email subject line. Replacement tokens may be used.'),
      '#required' => true,
    );
    $form['forward_email_defaults']['forward_email_message'] = array(
      '#type' => 'textarea',
      '#title' => t('Message body'),
      '#default_value' => $settings['forward_email_message'],
      '#rows' => 5,
      '#description' => t('Email message body. Replacement tokens may be used. The sender may be able to add their own personal message after this.'),
    );
    // Post processing filters
    $filter_options = array();
    $filter_options[''] = t('- None - ');
    foreach(filter_formats($this->currentUser()) as $key => $format) {
      $filter_options[$key] = $format->label();
    }
    if (count($filter_options) > 1) {
      $form['forward_filter_options'] = array(
        '#type' => 'details',
        '#title' => t('Filter'),
        '#open' => false,
      );
      $form['forward_filter_options']['forward_filter_format'] = array(
        '#type' => 'select',
        '#title' => t('Filter format'),
        '#default_value' => $settings['forward_filter_format'],
        '#options' => $filter_options,
        '#description' => t('Select a filter to apply to the forward message body. A filter with <a href="http://drupal.org/project/pathologic">Pathologic</a> assigned to it will convert relative links to absolute links. &nbsp;<a href="http://drupal.org/project/modules">More filters</a>.'),
      );
    }
    // Access Control
    $form['forward_access_control'] = array(
      '#type' => 'details',
      '#title' => t('Access Control'),
      '#open' => false,
      '#description' => t('The email build process normally uses anonymous visitor permissions to render the entity being forwarded.  This is appropriate for most sites.  If you bypass anonymous access control, and the person doing the forward is logged in, the permissions of the logged in account are used instead.  Bypassing anonymous access control creates a potential security risk because privileged information could be sent to people who are not authorized to view it.'),
    );
    $form['forward_access_control']['forward_bypass_access_control'] = array(
      '#type' => 'checkbox',
      '#title' => t('Bypass anonymous access control'),
      '#default_value' => $settings['forward_bypass_access_control'],
      '#description' => t('<em>Warning: selecting this option has security implications.</em>'),
    );
    // Flood Control
    $form['forward_flood_control_options'] = array(
      '#type' => 'details',
      '#title' => t('Flood Control'),
      '#open' => false,
    );
    $form['forward_flood_control_options']['forward_flood_control_limit'] = array(
      '#type' => 'select',
      '#title' => t('Flood control limit'),
      '#default_value' => $settings['forward_flood_control_limit'],
      '#options' => array('1' => '1', '5' => '5', '10' => '10', '15' => '15', '20' => '20', '25' => '25', '30' => '30', '35' => '35', '40' => '40', '50' => '50'),
      '#description' => t('How many times a user can use the form in a one hour period. This will help prevent the forward module from being used for spamming.'),
    );
    $form['forward_flood_control_options']['forward_flood_control_error'] = array(
      '#type' => 'textarea',
      '#title' => t('Flood control error'),
      '#default_value' => $settings['forward_flood_control_error'],
      '#rows' => 5,
      '#description' => t('This text appears if a user exceeds the flood control limit.  The value of the flood control limit setting will appear in place of @number in the message presented to users.'),
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate link icon path if provided
    if ($form_state->getValue('forward_link_icon')) {
      $image = entity_create('file');
      $image->setFileUri($form_state->getValue('forward_link_icon'));
      $image->setFilename(drupal_basename($image->getFileUri()));
      $errors = file_validate_is_image($image);
      if (count($errors)) {
        $form_state->setErrorByName('forward_link_icon', $this->t("The link icon path '@path' is invalid.", array('@path' => $form_state->getValue('forward_link_icon'))));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save entity types and bundles
    $values = $form_state->getValue('forward_entity_types');
    $values['node'] = TRUE;
    $entity_types = \Drupal::entityTypeManager()->getDefinitions();
    foreach ($entity_types as $type => $info) {
      if (!empty($values[$type])) {
        $this->config('forward.settings')->set('forward_entity_' . $type, TRUE);
      } else {
        $this->config('forward.settings')->set('forward_entity_' . $type, FALSE);
      }
      $bundles = \Drupal::entityManager()->getBundleInfo($type);
      foreach ($bundles as $bundle => $bundle_info) {
        $bundle_values = $form_state->getValue('forward_' . $type . '_types');
        if (!empty($values[$type]) && !empty($bundle_values[$bundle])) {
          $this->config('forward.settings')->set('forward_' . $type . '_' . $bundle, TRUE);
        } else {
          $this->config('forward.settings')->set('forward_' . $type . '_' . $bundle, FALSE);
        }
        // If only one bundle, it gets the same setting as its type
        if (count($bundles) == 1) {
          $this->config('forward.settings')->set('forward_' . $type . '_' . $bundle, $this->config('forward.settings')->get('forward_entity_' . $type));
        }
      }
    }
    // Save view modes
    $modes = array('full', 'teaser');
    $values = $form_state->getValue('forward_view_modes');
    foreach ($modes as $mode) {
      if (!empty($values[$mode])) {
        $this->config('forward.settings')->set('forward_view_' . $mode, TRUE);
      } else {
        $this->config('forward.settings')->set('forward_view_' . $mode, FALSE);
      }
    }
    // Save all other settings
    $this->config('forward.settings')
      ->set('forward_interface_type', $form_state->getValue('forward_interface_type'))
      ->set('forward_interface_title', $form_state->getValue('forward_interface_title'))
      ->set('forward_interface_weight', $form_state->getValue('forward_interface_weight'))
      ->set('forward_link_inline', $form_state->getValue('forward_link_inline'))
      ->set('forward_link_title', $form_state->getValue('forward_link_title'))
      ->set('forward_link_style', $form_state->getValue('forward_link_style'))
      ->set('forward_link_icon', $form_state->getValue('forward_link_icon'))
      ->set('forward_link_alias', $form_state->getValue('forward_link_alias'))
      ->set('forward_link_noindex', $form_state->getValue('forward_link_noindex'))
      ->set('forward_link_nofollow', $form_state->getValue('forward_link_nofollow'))
      ->set('forward_form_instructions', $form_state->getValue('forward_form_instructions'))
      ->set('forward_form_display_page', $form_state->getValue('forward_form_display_page'))
      ->set('forward_form_display_subject', $form_state->getValue('forward_form_display_subject'))
      ->set('forward_form_display_body', $form_state->getValue('forward_form_display_body'))
      ->set('forward_form_confirmation', $form_state->getValue('forward_form_confirmation'))
      ->set('forward_personal_message', $form_state->getValue('forward_personal_message'))
      ->set('forward_personal_message_filter', $form_state->getValue('forward_personal_message_filter'))
      ->set('forward_personal_message_tags', $form_state->getValue('forward_personal_message_tags'))
      ->set('forward_email_logo', $form_state->getValue('forward_email_logo'))
      ->set('forward_email_from_address', $form_state->getValue('forward_email_from_address'))
      ->set('forward_email_subject', $form_state->getValue('forward_email_subject'))
      ->set('forward_email_message', $form_state->getValue('forward_email_message'))
      ->set('forward_thankyou_send', $form_state->getValue('forward_thankyou_send'))
      ->set('forward_thankyou_subject', $form_state->getValue('forward_thankyou_subject'))
      ->set('forward_thankyou_text', $form_state->getValue('forward_thankyou_text'))
      ->set('forward_filter_format', $form_state->getValue('forward_filter_format'))
      ->set('forward_bypass_access_control', $form_state->getValue('forward_bypass_access_control'))
      ->set('forward_flood_control_limit', $form_state->getValue('forward_flood_control_limit'))
      ->set('forward_flood_control_error', $form_state->getValue('forward_flood_control_error'))
      ->save();

    parent::submitForm($form, $form_state);
  }
}
